/***************************************************************************
    kcdspreferences.cpp  -  preferences dialog
    ------------------------------------------------------------------------
    begin                : Mon Jun 10 2002
    copyright            : (C) 2002, 2005 by Tomas Horsky
    email                : tomas@riadoklan.sk
 ***************************************************************************/

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qtabbar.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qvbuttongroup.h>
#include <qwhatsthis.h>
#include <qlayout.h>
#include <qvbox.h>
#include <qgrid.h>
#include <klocale.h>
#include <klistbox.h>
#include <knuminput.h>
#include <kpushbutton.h>
#include "kcdssettings.h"

#include "kcdspreferences.h"

KcdsPreferences::KcdsPreferences(QWidget *parent, const char *name, bool modal, WFlags f):
	QTabDialog(parent, name, modal, f)
{
	setCaption(i18n("KCdSpeed preferences"));
	
	QWidget *tab1=new QWidget(this);
		QBoxLayout *lay1=new QVBoxLayout(tab1, 10);
		QVButtonGroup *group11=new QVButtonGroup(i18n("Device"), tab1);
			QGrid *grid111=new QGrid(2, Qt::Horizontal, group11);
			grid111->setSpacing(5);
				QLabel *lab1111=new QLabel(i18n("&Path to device:"), grid111);
				_device=new QLineEdit(grid111);
				lab1111->setBuddy(_device);
			_scsiDevice=new QCheckBox(i18n("SCSI &device configuration"), group11);
			connect(_scsiDevice, SIGNAL(toggled(bool)), SLOT(slotEnableScsiDevice(bool)));
			QGrid *grid112=new QGrid(6, Qt::Horizontal, group11);
			grid112->setSpacing(5);
				QLabel *lab1121=new QLabel(i18n("Scsi&Bus:"), grid112);
				_scsiBus=new KIntSpinBox(grid112);
				lab1121->setBuddy(_scsiBus);
				lab1121->setAlignment(Qt::AlignVCenter | Qt::AlignRight);
				QLabel *lab1122=new QLabel(i18n("&Target:"), grid112);
				_target=new KIntSpinBox(grid112);
				lab1122->setBuddy(_target);
				lab1122->setAlignment(Qt::AlignVCenter | Qt::AlignRight);
				QLabel *lab1123=new QLabel(i18n("&Lun:"), grid112);
				_lun=new KIntSpinBox(grid112);
				lab1123->setBuddy(_lun);
				lab1123->setAlignment(Qt::AlignVCenter | Qt::AlignRight);
		lay1->addWidget(group11);
		QWhatsThis::add(group11, i18n("<P>Device settings.<DL>"
		                              "<DT><B>Path to device</B><DD>This is the path to CD-ROM device controlled by this applet."
		                              "<DT><B>SCSI device configuration</B><DD>If you want to use <EM>cdrecord</EM> to eject and close tray, you may want to specify <B>ScsiBus</B>, <B>Target</B> and <B>Lun</B> for your drive. If not checked, default device will be used (if any)."
		                              "</DL></P>"));
		QVButtonGroup *group12=new QVButtonGroup(i18n("Misc"), tab1);
			_confirmSpeed=new QCheckBox(i18n("&Confirm speed change"), group12);
			_ignoreErrors=new QCheckBox(i18n("&Ignore eject, close, and speed change errors"), group12);
		lay1->addWidget(group12);
		QWhatsThis::add(group12, i18n("<P>Miscellaneous settings.<DL>"
		                              "<DT><B>Confirm speed change</B><DD>If checked, information dialog will pop up after speed was successfully changed."
		                              "<DT><B>Ignore eject, close, and speed change errors</B><DD>If checked and attempt to eject, close, or change speed causes error, applet will not pop up error message box."
		                              "</DL></P>"));
	addTab(tab1, i18n("&General"));
	
	QWidget *tab2=new QWidget(this);
		QBoxLayout *lay2=new QVBoxLayout(tab2, 10);
		QGroupBox *group21=new QGroupBox(2, Qt::Horizontal, i18n("Available speeds"), tab2);
			_availSpeeds=new KListBox(group21);
			QVBox *vbox211=new QVBox(group21);
			vbox211->setSpacing(5);
				_speedSpin=new KIntSpinBox(vbox211);
				_speedSpin->setMinValue(1);
				_speedSpin->setSuffix("x");
				KPushButton *butt2111=new KPushButton(i18n("&Add"), vbox211);
				connect(butt2111, SIGNAL(clicked()), SLOT(slotSpeedsAdd()));
				KPushButton *butt2112=new KPushButton(i18n("&Remove"), vbox211);
				connect(butt2112, SIGNAL(clicked()), SLOT(slotSpeedsRemove()));
				KPushButton *butt2113=new KPushButton(i18n("&Detect"), vbox211);
				connect(butt2113, SIGNAL(clicked()), SLOT(slotSpeedsDetect()));
				butt2113->setEnabled(false);	//TODO
		lay2->addWidget(group21);
		QWhatsThis::add(group21, i18n("<P>This section shows speeds, that are available in applet's selection box. Not all speeds are supported by every device, so try which speeds are suitable for you.<DL>"
		                              "<DT><B>Add</B><DD>This will add value from spin box to list of available speeds."
		                              "<DT><B>Remove</B><DD>This will remove highlighted value from list of available speeds."
		                              "<DT><B>Detect</B><DD>This is not yet implemented."
		                              "</DL></P>"));
		QButtonGroup *group22=new QButtonGroup(2, Qt::Horizontal, i18n("Startup"), tab2);
			_startupMinimum=new QRadioButton(i18n("Set m&inimum speed"), group22);
			_startupMaximum=new QRadioButton(i18n("Set ma&ximum speed"), group22);
			_startupLast=new QRadioButton(i18n("Set &last used speed"), group22);
			_startupNothing=new QRadioButton(i18n("Do &not change speed"), group22);
		lay2->addWidget(group22);
		QWhatsThis::add(group22, i18n("<P>Here you can choose which speed will be set when applet starts. If applet does not change speed, or current speed is not available in selection box, it will show '?' as current speed.<DL>"
		                              "<DT><B>Set minimum speed</B><DD>This chooses minimum value from available speeds, and sets CD-ROM to this speed."
		                              "<DT><B>Set maximum speed</B><DD>This chooses maximum value from available speeds, and sets CD-ROM to this speed."
		                              "<DT><B>Set last used speed</B><DD>This sets CD-ROM to speed, that was most recently selected from selection box."
		                              "<DT><B>Do not change speed</B><DD>This leaves speed untouched."
		                              "</DL></P>"));
	addTab(tab2, i18n("&Speeds"));
	
	QWidget *tab3=new QWidget(this);
		QBoxLayout *lay3=new QVBoxLayout(tab3, 10);
		QVButtonGroup *group31=new QVButtonGroup(i18n("Eject && Close commands"), tab3);
			_ejectIoctl=new QRadioButton(i18n("Use internal &ioctl function"), group31);
			_ejectCdrecord=new QRadioButton(i18n("Use cd&record"), group31);
			_ejectOther=new QRadioButton(i18n("Use &other command"), group31);
			connect(_ejectOther, SIGNAL(toggled(bool)), SLOT(slotEnableOtherCommand(bool)));
			QGrid *grid311=new QGrid(2, Qt::Horizontal, group31);
			grid311->setSpacing(5);
				QLabel *lab3111=new QLabel(i18n("E&ject command:"), grid311);
				_otherEject=new QLineEdit(grid311);
				lab3111->setBuddy(_otherEject);
				QLabel *lab3112=new QLabel(i18n("C&lose command:"), grid311);
				_otherClose=new QLineEdit(grid311);
				lab3112->setBuddy(_otherClose);
		lay3->addWidget(group31);
		QWhatsThis::add(group31, i18n("<P>Choose the way of ejecting and closing CD-ROM tray. It is possible that something will not work, so you can try different ways to do the same thing.<DL>"
		                              "<DT><B>Use internal ioctl function</B><DD>Internal low-level function, that should work for most devices."
		                              "<DT><B>Use cdrecord</B><DD>If you have installed and correctly configured this CD-burning application, you can try it. <EM>WARNING: it may be possible to eject tray with cdrecord while CD is mounted !</EM>"
		                              "<DT><B>Use other command</B><DD>If you have a tool, that is known to work properly, you can use it. You have to write both commands for ejecting and closing tray. In commands you can use next variables:<UL>"
		                              "	<LI><TT>$DEVICE</TT> - will be replaced by <B>General::Device::Path to device</B>"
		                              "	<LI><TT>$SCSIBUS</TT> - will be replaced by <B>General::Device::ScsiBus</B>"
		                              "	<LI><TT>$TARGET</TT> - will be replaced by <B>General::Device::Target</B>"
		                              "	<LI><TT>$LUN</TT> - will be replaced by <B>General::Device::Lun</B>"
		                              "</UL>Example: command to eject using cdrecord looks like this:<BR><TT>cdrecord dev=$SCSIBUS,$TARGET,$LUN -eject</TT>"
		                              "</DL></P>"));
	addTab(tab3, i18n("&Eject && Close"));
	
	QWidget *tab4=new QWidget(this);
		QBoxLayout *lay4=new QVBoxLayout(tab4, 10);
		QVButtonGroup *group41=new QVButtonGroup(i18n("Mount && Unmount options"), tab4);
			_customMountPoint=new QCheckBox(i18n("&Use custom mount point"), group41);
			connect(_customMountPoint, SIGNAL(toggled(bool)), SLOT(slotEnableMountPoint(bool)));
			QGrid *grid411=new QGrid(2, Qt::Horizontal, group41);
			grid411->setSpacing(5);
				QLabel *lab4111=new QLabel(i18n("Mount &point:"), grid411);
				_mountPoint=new QLineEdit(grid411);
				lab4111->setBuddy(_mountPoint);
			_openFilemanager=new QCheckBox(i18n("&Open filemanager on mount"), group41);
		lay4->addWidget(group41);
		QWhatsThis::add(group41, i18n("<P>You can customize mounting and unmounting process here.<DL>"
		                              "<DT><B>Use custom mount point</B><DD>If checked, CD will be mounted to directory given by <B>Mount point</B>. If not, <EM>fstab</EM> default will be used instead. <EM>WARNING: you will probably need root privileges to use different mount point !</EM>"
		                              "<DT><B>Open filemanager on mount</B><DD>If checked, filemanager window will be opened after successful mount."
		                              "</DL></P>"));
		QVButtonGroup *group42=new QVButtonGroup(i18n("Automatic mounting && unmounting"), tab4);
			_autoMount=new QCheckBox(i18n("Mount &after closing the tray"), group42);
			_autoUnmount=new QCheckBox(i18n("Unmount &before ejecting the tray"), group42);
		lay4->addWidget(group42);
		QWhatsThis::add(group42, i18n("<P>This section lets you turn on and off automatic mounting and unmounting.<DL>"
		                              "<DT><B>Mount after closing the tray</B><DD>If checked, applet attempts to automatically mount CD after closing tray."
		                              "<DT><B>Unmount before ejecting the tray</B><DD>If checked, applet will automatically unmount CD before ejecting tray, if it is mounted."
		                              "</DL></P>"));
	addTab(tab4, i18n("&Mount && Unmount"));
	
	setCancelButton();
	setApplyButton();
	tab1->setMinimumWidth(370);	//if I set this->setMinimumWidth(), it ignores me, so this does the work
}

KcdsPreferences::~KcdsPreferences()
{
}

void KcdsPreferences::readSettings(KcdsSettings *settings)
{
	//tab1
	_device->setText(settings->device());
	
	_scsiDevice->setChecked(settings->scsiDevice());
	slotEnableScsiDevice(_scsiDevice->isChecked());
	_scsiBus->setValue(settings->scsiBus().toInt());
	_target->setValue(settings->target().toInt());
	_lun->setValue(settings->lun().toInt());
	
	_confirmSpeed->setChecked(settings->confirmSpeed());
	_ignoreErrors->setChecked(settings->ignoreErrors());
	
	//tab2
	_availSpeeds->insertStringList(settings->availSpeeds());
	
	switch(settings->startup()) {
		case KcdsSettings::Minimum: _startupMinimum->setChecked(true); break;
		case KcdsSettings::Maximum: _startupMaximum->setChecked(true); break;
		case KcdsSettings::Last:    _startupLast->setChecked(true);    break;
		case KcdsSettings::Nothing: _startupNothing->setChecked(true); break;
	}
	
	//tab3
	switch(settings->ejectClose()) {
		case KcdsSettings::Ioctl:    _ejectIoctl->setChecked(true);    break;
		case KcdsSettings::Cdrecord: _ejectCdrecord->setChecked(true); break;
		case KcdsSettings::Other:    _ejectOther->setChecked(true);    break;
	}
	slotEnableOtherCommand(_ejectOther->isChecked());
	_otherEject->setText(settings->otherEject());
	_otherClose->setText(settings->otherClose());
	
	//tab4
	_customMountPoint->setChecked(settings->customMountPoint());
	slotEnableMountPoint(_customMountPoint->isChecked());
	_mountPoint->setText(settings->mountPoint());
	_openFilemanager->setChecked(settings->openFilemanager());
	
	_autoMount->setChecked(settings->autoMount());
	_autoUnmount->setChecked(settings->autoUnmount());
}

void KcdsPreferences::writeSettings(KcdsSettings *settings)
{
	//tab1
	settings->setDevice(_device->text());
	
	settings->setScsiDevice(_scsiDevice->isChecked());
	settings->setScsiBus(_scsiBus->text());
	settings->setTarget(_target->text());
	settings->setLun(_lun->text());
	
	settings->setConfirmSpeed(_confirmSpeed->isChecked());
	settings->setIgnoreErrors(_ignoreErrors->isChecked());
	
	//tab2
	QStringList as;
	for(unsigned i=0; i<_availSpeeds->count(); i++)
		as+=_availSpeeds->text(i);
	settings->setAvailSpeeds(as);
	
	KcdsSettings::Startup s;
	if(_startupMinimum->isChecked())
		s=KcdsSettings::Minimum;
	else if(_startupMaximum->isChecked())
		s=KcdsSettings::Maximum;
	else if(_startupLast->isChecked())
		s=KcdsSettings::Last;
	else
		s=KcdsSettings::Nothing;
	settings->setStartup(s);
	
	//tab3
	KcdsSettings::EjectClose e;
	if(_ejectIoctl->isChecked())
		e=KcdsSettings::Ioctl;
	else if(_ejectCdrecord->isChecked())
		e=KcdsSettings::Cdrecord;
	else
		e=KcdsSettings::Other;
	settings->setEjectClose(e);
	settings->setOtherEject(_otherEject->text());
	settings->setOtherClose(_otherClose->text());
	
	//tab4
	settings->setCustomMountPoint(_customMountPoint->isChecked());
	settings->setMountPoint(_mountPoint->text());
	settings->setOpenFilemanager(_openFilemanager->isChecked());
	
	settings->setAutoMount(_autoMount->isChecked());
	settings->setAutoUnmount(_autoUnmount->isChecked());
}

void KcdsPreferences::slotEnableScsiDevice(bool on)
{
	_scsiBus->setEnabled(on);
	_target->setEnabled(on);
	_lun->setEnabled(on);
}

void KcdsPreferences::slotSpeedsAdd()
{
	//check if there is not already that speed
	for(unsigned i=0; i<_availSpeeds->count(); i++)
		if(_availSpeeds->text(i)==_speedSpin->text())
			return;
	_availSpeeds->insertItem(_speedSpin->text());
}

void KcdsPreferences::slotSpeedsRemove()
{
	if(_availSpeeds->currentItem()>=0)
		_availSpeeds->removeItem(_availSpeeds->currentItem());
}

void KcdsPreferences::slotSpeedsDetect()
{
	//TODO
	_availSpeeds->clear();
}

void KcdsPreferences::slotEnableOtherCommand(bool on)
{
	_otherEject->setEnabled(on);
	_otherClose->setEnabled(on);
}

void KcdsPreferences::slotEnableMountPoint(bool on)
{
	_mountPoint->setEnabled(on);
}
