/***************************************************************************
    kcdssettings.cpp  -  settings
    ------------------------------------------------------------------------
    begin                : Sat Aug 31 2002
    copyright            : (C) 2002, 2005 by Tomas Horsky
    email                : tomas@riadoklan.sk
 ***************************************************************************/

#include <kconfig.h>

#include "kcdssettings.h"

#define CONF_LASTSPEED "LastSpeed"
#define CONF_LOCKBUTTONS "LockButtons"
#define CONF_DEVICE "Device"
#define CONF_SCSIDEVICE "ScsiDevice"
#define CONF_SCSIBUS "ScsiBus"
#define CONF_TARGET "Target"
#define CONF_LUN "Lun"
#define CONF_CONFIRMSPEED "ConfirmSpeed"
#define CONF_IGNOREERRORS "IgnoreErrors"
#define CONF_AVAILSPEEDS "AvailSpeeds"
#define CONF_STARTUP "Startup"
#	define CONF_STARTUP_MINIMUM "minimum"
#	define CONF_STARTUP_MAXIMUM "maximum"
#	define CONF_STARTUP_LAST "last"
#	define CONF_STARTUP_NOTHING "nothing"
#define CONF_EJECTCLOSE "EjectClose"
#	define CONF_EJECTCLOSE_IOCTL "ioctl"
#	define CONF_EJECTCLOSE_CDRECORD "cdrecord"
#	define CONF_EJECTCLOSE_OTHER "other"
#define CONF_OTHEREJECT "OtherEject"
#define CONF_OTHERCLOSE "OtherClose"
#define CONF_CUSTOMMOUNTPOINT "CustomMountPoint"
#define CONF_MOUNTPOINT "MountPoint"
#define CONF_OPENFILEMANAGER "OpenFilemanager"
#define CONF_AUTOMOUNT "AutoMount"
#define CONF_AUTOUNMOUNT "AutoUnmount"

KcdsSettings::KcdsSettings(KConfig *conf):
	_config(conf)
{
	readConfig();
}

KcdsSettings::~KcdsSettings()
{
	writeConfig();
}

void KcdsSettings::readConfig()
{
	_lastSpeed=_config->readEntry(CONF_LASTSPEED);
	_lockButtons=_config->readBoolEntry(CONF_LOCKBUTTONS, false);
	
	//tab1
	_device=_config->readEntry(CONF_DEVICE, QString("/dev/cdrom"));
	
	_scsiDevice=_config->readBoolEntry(CONF_SCSIDEVICE, false);
	_scsiBus=_config->readEntry(CONF_SCSIBUS, QString("0"));
	_target=_config->readEntry(CONF_TARGET, QString("0"));
	_lun=_config->readEntry(CONF_LUN, QString("0"));
	
	_confirmSpeed=_config->readBoolEntry(CONF_CONFIRMSPEED, false);
	_ignoreErrors=_config->readBoolEntry(CONF_IGNOREERRORS, false);
	
	//tab2
	if(!_config->hasKey(CONF_AVAILSPEEDS)) {
		_availSpeeds+=QString("4x");
		_availSpeeds+=QString("8x");
		_availSpeeds+=QString("52x");
	} else
		_availSpeeds=_config->readListEntry(CONF_AVAILSPEEDS);
	
	QString stup(_config->readEntry(CONF_STARTUP, QString(CONF_STARTUP_NOTHING)));
	if(stup==CONF_STARTUP_MINIMUM)
		_startup=Minimum;
	else if(stup==CONF_STARTUP_MAXIMUM)
		_startup=Maximum;
	else if(stup==CONF_STARTUP_LAST)
		_startup=Last;
	else
		_startup=Nothing;
	
	//tab3
	QString ejct(_config->readEntry(CONF_EJECTCLOSE, QString(CONF_EJECTCLOSE_IOCTL)));
	if(ejct==CONF_EJECTCLOSE_IOCTL)
		_ejectClose=Ioctl;
	else if(ejct==CONF_EJECTCLOSE_CDRECORD)
		_ejectClose=Cdrecord;
	else
		_ejectClose=Other;
	_otherEject=_config->readEntry(CONF_OTHEREJECT, QString("eject -r $DEVICE"));
	_otherClose=_config->readEntry(CONF_OTHERCLOSE, QString("eject -t $DEVICE"));
	
	//tab4
	_customMountPoint=_config->readBoolEntry(CONF_CUSTOMMOUNTPOINT, false);
	_mountPoint=_config->readEntry(CONF_MOUNTPOINT);
	_openFilemanager=_config->readBoolEntry(CONF_OPENFILEMANAGER, false);
	
	_autoMount=_config->readBoolEntry(CONF_AUTOMOUNT, false);
	_autoUnmount=_config->readBoolEntry(CONF_AUTOUNMOUNT, false);
}

void KcdsSettings::writeConfig()
{
	_config->writeEntry(CONF_LASTSPEED, _lastSpeed);
	_config->writeEntry(CONF_LOCKBUTTONS, _lockButtons);
	
	//tab1
	_config->writeEntry(CONF_DEVICE, _device);
	
	_config->writeEntry(CONF_SCSIDEVICE, _scsiDevice);
	_config->writeEntry(CONF_SCSIBUS, _scsiBus);
	_config->writeEntry(CONF_TARGET, _target);
	_config->writeEntry(CONF_LUN, _lun);
	
	_config->writeEntry(CONF_CONFIRMSPEED, _confirmSpeed);
	_config->writeEntry(CONF_IGNOREERRORS, _ignoreErrors);
	
	//tab2
	_config->writeEntry(CONF_AVAILSPEEDS, _availSpeeds);
	
	QString stup;
	switch(_startup) {
		case Minimum: stup=CONF_STARTUP_MINIMUM; break;
		case Maximum: stup=CONF_STARTUP_MAXIMUM; break;
		case Last:    stup=CONF_STARTUP_LAST;    break;
		case Nothing: stup=CONF_STARTUP_NOTHING; break;
	}
	_config->writeEntry(CONF_STARTUP, stup);
	
	//tab3
	QString ejct;
	switch(_ejectClose) {
		case Ioctl:    ejct=CONF_EJECTCLOSE_IOCTL;    break;
		case Cdrecord: ejct=CONF_EJECTCLOSE_CDRECORD; break;
		case Other:    ejct=CONF_EJECTCLOSE_OTHER;    break;
	}
	_config->writeEntry(CONF_EJECTCLOSE, ejct);
	_config->writeEntry(CONF_OTHEREJECT, _otherEject);
	_config->writeEntry(CONF_OTHERCLOSE, _otherClose);
	
	//tab4
	_config->writeEntry(CONF_CUSTOMMOUNTPOINT, _customMountPoint);
	_config->writeEntry(CONF_MOUNTPOINT, _mountPoint);
	_config->writeEntry(CONF_OPENFILEMANAGER, _openFilemanager);
	
	_config->writeEntry(CONF_AUTOMOUNT, _autoMount);
	_config->writeEntry(CONF_AUTOUNMOUNT, _autoUnmount);
	
	_config->sync();
}
