/***************************************************************************
    kcdssettings.h  -  settings
    ------------------------------------------------------------------------
    begin                : Sat Aug 31 2002
    copyright            : (C) 2002, 2005 by Tomas Horsky
    email                : tomas@riadoklan.sk
 ***************************************************************************/

#ifndef KCDSSETTINGS_H
#define KCDSSETTINGS_H

#include <qstring.h>
#include <qstringlist.h>

class KConfig;

/**	Class that holds configuration settings.
	Hm, maybe this should be only interface for KConfig methods -
	do not keep data here, but only call readEntry(SOME_KEY), and
	writeEntry(SOME_KEY)...
*/
class KcdsSettings
{
	public:
		enum Startup {
			Minimum,
			Maximum,
			Last,
			Nothing
		};
		enum EjectClose {
			Ioctl,
			Cdrecord,
			Other
		};
		
	public:
		/**	Constructor
		*/
		KcdsSettings(KConfig *conf);
		/**	Destructor
		*/
		virtual ~KcdsSettings();
		
		/**	Reads configuration, from KConfig object.
		*/
		void readConfig();
		/**	Writes configuration to KConfig object, and to disk.
		*/
		void writeConfig();
		
		void setLastSpeed(const QString &s) {_lastSpeed=s;}
		void setLockButtons(bool l) {_lockButtons=l;}
		void setDevice(const QString &d) {_device=d;}
		void setScsiDevice(bool s) {_scsiDevice=s;}
		void setScsiBus(const QString &s) {_scsiBus=s;}
		void setTarget(const QString &t) {_target=t;}
		void setLun(const QString &l) {_lun=l;}
		void setConfirmSpeed(bool c) {_confirmSpeed=c;}
		void setIgnoreErrors(bool i) {_ignoreErrors=i;}
		void setAvailSpeeds(const QStringList &a) {_availSpeeds=a;}
		void setStartup(Startup s) {_startup=s;}
		void setEjectClose(EjectClose e) {_ejectClose=e;}
		void setOtherEject(const QString &o) {_otherEject=o;}
		void setOtherClose(const QString &o) {_otherClose=o;}
		void setCustomMountPoint(bool c) {_customMountPoint=c;}
		void setMountPoint(const QString &m) {_mountPoint=m;}
		void setOpenFilemanager(bool o) {_openFilemanager=o;}
		void setAutoMount(bool a) {_autoMount=a;}
		void setAutoUnmount(bool a) {_autoUnmount=a;}
		
		QString lastSpeed() const {return _lastSpeed;}
		bool lockButtons() const {return _lockButtons;}
		QString device() const {return _device;}
		bool scsiDevice() const {return _scsiDevice;}
		QString scsiBus() const {return _scsiBus;}
		QString target() const {return _target;}
		QString lun() const {return _lun;}
		bool confirmSpeed() const {return _confirmSpeed;}
		bool ignoreErrors() const {return _ignoreErrors;}
		QStringList availSpeeds() const {return _availSpeeds;}
		Startup startup() const {return _startup;}
		EjectClose ejectClose() const {return _ejectClose;}
		QString otherEject() const {return _otherEject;}
		QString otherClose() const {return _otherClose;}
		bool customMountPoint() const {return _customMountPoint;}
		QString mountPoint() const {return _mountPoint;}
		bool openFilemanager() const {return _openFilemanager;}
		bool autoMount() const {return _autoMount;}
		bool autoUnmount() const {return _autoUnmount;}
		
	protected:
		KConfig *_config;
		
		QString _lastSpeed;
		bool _lockButtons;
		QString _device;
		bool _scsiDevice;
		QString _scsiBus;
		QString _target;
		QString _lun;
		bool _confirmSpeed;
		bool _ignoreErrors;
		QStringList _availSpeeds;
		Startup _startup;
		EjectClose _ejectClose;
		QString _otherEject;
		QString _otherClose;
		bool _customMountPoint;
		QString _mountPoint;
		bool _openFilemanager;
		bool _autoMount;
		bool _autoUnmount;
};

#endif //KCDSSETTINGS_H
